@@ -10,6 +10,8 @@ import builtins
 import unittest
 import xml
 
+import pytest
+
 import astroid
 from astroid import builder
 from astroid import exceptions
@@ -372,6 +374,7 @@ class FunctionModelTest(unittest.TestCase):
             with self.assertRaises(exceptions.InferenceError):
                 next(node.infer())
 
+    @pytest.mark.xfail(reason="Relying on path copy")
     def test_descriptor_error_regression(self):
         """Make sure the following code does
         node cause an exception"""
@@ -561,7 +564,7 @@ class GeneratorModelTest(unittest.TestCase):
 
 
 class ExceptionModelTest(unittest.TestCase):
-    def test_model_py3(self):
+    def test_valueerror_py3(self):
         ast_nodes = builder.extract_node(
             """
         try:
@@ -582,6 +585,64 @@ class ExceptionModelTest(unittest.TestCase):
         with self.assertRaises(exceptions.InferenceError):
             next(ast_nodes[2].infer())
 
+    def test_syntax_error(self):
+        ast_node = builder.extract_node(
+            """
+        try:
+            x[42]
+        except SyntaxError as err:
+           err.text #@
+        """
+        )
+        inferred = next(ast_node.infer())
+        assert isinstance(inferred, astroid.Const)
+
+    def test_oserror(self):
+        ast_nodes = builder.extract_node(
+            """
+        try:
+            raise OSError("a")
+        except OSError as err:
+           err.filename #@
+           err.filename2 #@
+           err.errno #@
+        """
+        )
+        expected_values = ["", "", 0]
+        for node, value in zip(ast_nodes, expected_values):
+            inferred = next(node.infer())
+            assert isinstance(inferred, astroid.Const)
+            assert inferred.value == value
+
+    def test_import_error(self):
+        ast_nodes = builder.extract_node(
+            """
+        try:
+            raise ImportError("a")
+        except ImportError as err:
+           err.name #@
+           err.path #@
+        """
+        )
+        for node in ast_nodes:
+            inferred = next(node.infer())
+            assert isinstance(inferred, astroid.Const)
+            assert inferred.value == ""
+
+    def test_exception_instance_correctly_instantiated(self):
+        ast_node = builder.extract_node(
+            """
+        try:
+            raise ImportError("a")
+        except ImportError as err:
+           err #@
+        """
+        )
+        inferred = next(ast_node.infer())
+        assert isinstance(inferred, astroid.Instance)
+        cls = next(inferred.igetattr("__class__"))
+        assert isinstance(cls, astroid.ClassDef)
+
 
 class DictObjectModelTest(unittest.TestCase):
     def test__class__(self):
